@@ -5,7 +5,6 @@ import logging
 import time
 from .inout import CravatReader, CravatWriter, AllMappingsParser
 from .constants import crx_def, crx_idx, crg_def, crg_idx, crt_def, crt_idx, gene_level_so_exclude
-from .util import most_severe_so, so_severity
 from .exceptions import InvalidData
 from cravat.config_loader import ConfigLoader
 import sys
@@ -16,37 +15,18 @@ from types import SimpleNamespace
 import multiprocessing as mp
 import cravat.admin_util as au
 import time
+import cravat.util
 
 class BaseMapper(object):
     """
     BaseMapper is the parent class for Cravat Mapper objects.
-    It recieves a crv file and writes crx and crg files based on it's child
+    It receives a crv file and writes crx and crg files based on it's child
     mapper's map() function.
     It handles command line arguments, option parsing and file io for the
     mapping process.
     """
-    def __init__(self, cmd_args, status_writer, live=False):
-        self.t = time.time()
-        if live:
-            self.live = live
-            self.cmd_args = SimpleNamespace()
-            self.cmd_args.include_sources = []
-            self.cmd_args.exclude_sources = []
-            self.cmd_args.primary_source = 'EnsemblT'
-            self.input_path = ''
-            self._setup_logger()
-            return
-        self.status_writer = status_writer
-        main_fpath = cmd_args[0]
-        main_basename = os.path.basename(main_fpath)
-        if '.' in main_basename:
-            self.module_name = '.'.join(main_basename.split('.')[:-1])
-        else:
-            self.module_name = main_basename
-        self.module_dir = os.path.dirname(main_fpath)
-        self.mapper_dir = os.path.dirname(main_fpath)
+    def __init__(self, *inargs, **inkwargs):
         self.cmd_parser = None
-        self.cmd_args = None
         self.input_path = None
         self.input_dir = None
         self.reader = None
@@ -58,13 +38,32 @@ class BaseMapper(object):
         self.crx_writer = None
         self.crg_writer = None
         self.crt_writer = None
-        self.gene_sources = []
-        self.primary_gene_source = None
-        self.gene_info = {}
-        self.written_primary_transc = set([])
         self._define_main_cmd_args()
         self._define_additional_cmd_args()
-        self._parse_cmd_args(cmd_args)
+        self._parse_cmd_args(inargs, inkwargs)
+        if hasattr(self.args, 'status_writer') == False:
+            status_writer = None
+        else:
+            status_writer = self.args.status_writer
+        if hasattr(self.args, 'live') == False:
+            live = False
+        else:
+            live = self.args.live
+        self.live = live
+        self.t = time.time()
+        self.status_writer = status_writer
+        main_fpath = self.args.script_path
+        main_basename = os.path.basename(main_fpath)
+        if '.' in main_basename:
+            self.module_name = '.'.join(main_basename.split('.')[:-1])
+        else:
+            self.module_name = main_basename
+        self.module_dir = os.path.dirname(main_fpath)
+        self.mapper_dir = os.path.dirname(main_fpath)
+        self.gene_sources = []
+        #self.primary_gene_source = None
+        self.gene_info = {}
+        #self.written_primary_transc = set([])
         self._setup_logger()
         config_loader = ConfigLoader()
         self.conf = config_loader.get_module_conf(self.module_name)
@@ -72,9 +71,9 @@ class BaseMapper(object):
 
     def _define_main_cmd_args(self):
         self.cmd_parser = argparse.ArgumentParser()
-        self.cmd_parser.add_argument('path',
-                                    help='Path to this mapper\'s python module')
-        self.cmd_parser.add_argument('input',
+        #self.cmd_parser.add_argument('path',
+        #                            help='Path to this mapper\'s python module')
+        self.cmd_parser.add_argument('input_file',
                                      help='Input crv file')
         self.cmd_parser.add_argument('-n',
                                      dest='name',
@@ -105,34 +104,42 @@ class BaseMapper(object):
             dest='postfix',
             default='',
             help=argparse.SUPPRESS)
+        self.cmd_parser.add_argument('--primary-transcript',
+            dest='primary_transcript',
+            nargs='*',
+            default=['mane'],
+            help='"mane" for MANE transcripts as primary transcripts, or a path to a file of primary transcripts. MANE is default.')
 
     def _define_additional_cmd_args(self):
         """This method allows sub-classes to override and provide addittional command line args"""
         pass
 
-    def _parse_cmd_args(self, args):
-        self.cmd_args = self.cmd_parser.parse_args(args)
-        self.input_path = os.path.abspath(self.cmd_args.input)
+    def _parse_cmd_args(self, inargs, inkwargs):
+        args = cravat.util.get_args(self.cmd_parser, inargs, inkwargs)
+        self.input_path = os.path.abspath(args.input_file)
         self.input_dir, self.input_fname = os.path.split(self.input_path)
-        if self.cmd_args.output_dir:
-            self.output_dir = self.cmd_args.output_dir
+        if args.output_dir:
+            self.output_dir = args.output_dir
         else:
             self.output_dir = self.input_dir
         if not(os.path.exists(self.output_dir)):
             os.makedirs(self.output_dir)
-        if self.cmd_args.name:
-            self.output_base_fname = self.cmd_args.name
+        if hasattr(args, 'run_name'):
+            self.output_base_fname = args.run_name
         else:
             self.output_base_fname = self.input_fname
         self.confs = None
-        if self.cmd_args.confs is not None:
-            confs = self.cmd_args.confs.lstrip('\'').rstrip('\'').replace("'", '"')
+        if args.confs is not None:
+            confs = args.confs.lstrip('\'').rstrip('\'').replace("'", '"')
             self.confs = json.loads(confs)
-        self.slavemode = self.cmd_args.slavemode
-        self.postfix = self.cmd_args.postfix
+        self.slavemode = args.slavemode
+        self.postfix = args.postfix
+        self.primary_transcript_paths = args.primary_transcript
+        self.args = args
 
     def base_setup(self):
-        self._setup_io()
+        if self.live == False:
+            self._setup_io()
         self.setup()
 
     def setup(self):
@@ -154,8 +161,8 @@ class BaseMapper(object):
         output. Open plain file for err output.
         """
         # Reader
-        if self.cmd_args.seekpos is not None and self.cmd_args.chunksize is not None:
-            self.reader = CravatReader(self.input_path, seekpos=int(self.cmd_args.seekpos), chunksize=int(self.cmd_args.chunksize))
+        if self.args.seekpos is not None and self.args.chunksize is not None:
+            self.reader = CravatReader(self.input_path, seekpos=int(self.args.seekpos), chunksize=int(self.args.chunksize))
         else:
             self.reader = CravatReader(self.input_path)
         # Various output files
@@ -177,6 +184,10 @@ class BaseMapper(object):
         self.crx_writer.write_meta_line('title', self.conf['title'])
         self.crx_writer.write_meta_line('version', self.conf['version'])
         self.crx_writer.write_meta_line('modulename', self.module_name)
+        if self.primary_transcript_paths is None:
+            self.crx_writer.write_meta_line('primary_transcript_paths', '')
+        else:
+            self.crx_writer.write_meta_line('primary_transcript_paths', ','.join(self.primary_transcript_paths))
         # .crg
         crg_fname = '.'.join(output_toks) + '.crg'
         self.crg_path = os.path.join(self.output_dir, crg_fname)
@@ -212,7 +223,6 @@ class BaseMapper(object):
         count = 0
         last_status_update_time = time.time()
         crx_data = None
-        alt_transcripts = None
         output = {}
         for ln, line, crv_data in self.reader.loop_data():
             crx_data = None
@@ -223,7 +233,11 @@ class BaseMapper(object):
                     if count % 10000 == 0 or cur_time - last_status_update_time > 3:
                         self.status_writer.queue_status_update('status', 'Running gene mapper: line {}'.format(count))
                         last_status_update_time = cur_time
-                crx_data, alt_transcripts = self.map(crv_data)
+                if crv_data['alt_base'] == '*':
+                    crx_data = crv_data
+                    crx_data['all_mappings'] = '{}'
+                else:
+                    crx_data = self.map(crv_data)
                 # Skip cases where there was no change. Can result if ref_base not in original input
                 if crx_data['ref_base'] == crx_data['alt_base']:
                     continue
@@ -233,8 +247,6 @@ class BaseMapper(object):
             if crx_data is not None:
                 self.crx_writer.write_data(crx_data)
                 self._add_crx_to_gene_info(crx_data)
-            if alt_transcripts is not None:
-                self._write_to_crt(alt_transcripts)
         self._write_crg()
         stop_time = time.time()
         self.logger.info('finished: %s' \
@@ -254,13 +266,15 @@ class BaseMapper(object):
         self.base_setup()
         start_time = time.time()
         tstamp = time.asctime(time.localtime(start_time))
-        self.logger.info(f'started: {tstamp} | {self.cmd_args.seekpos}')
+        self.logger.info(f'started: {tstamp} | {self.args.seekpos}')
         if self.status_writer is not None:
-            self.status_writer.queue_status_update('status', 'Started {} ({})'.format(self.conf['title'], self.module_name))
+            self.status_writer.queue_status_update(
+                'status', 
+                'Started {} ({})'.format(self.conf['title'], 
+                self.module_name))
         count = 0
         last_status_update_time = time.time()
         crx_data = None
-        alt_transcripts = None
         for ln, line, crv_data in self.reader.loop_data():
             try:
                 count += 1
@@ -269,7 +283,11 @@ class BaseMapper(object):
                     if count % 10000 == 0 or cur_time - last_status_update_time > 3:
                         self.status_writer.queue_status_update('status', 'Running gene mapper: line {}'.format(count))
                         last_status_update_time = cur_time
-                crx_data, alt_transcripts = self.map(crv_data)
+                if crv_data['alt_base'] == '*':
+                    crx_data = crv_data
+                    crx_data['all_mappings'] = '{}'
+                else:
+                    crx_data = self.map(crv_data)
                 if crx_data is None:
                     continue
                 # Skip cases where there was no change. Can result if ref_base not in original input
@@ -280,12 +298,10 @@ class BaseMapper(object):
             if crx_data is not None:
                 self.crx_writer.write_data(crx_data)
                 self._add_crx_to_gene_info(crx_data)
-            if alt_transcripts is not None:
-                self._write_to_crt(alt_transcripts)
         self._write_crg()
         stop_time = time.time()
         tstamp = time.asctime(time.localtime(stop_time))
-        self.logger.info(f'finished: {tstamp} | {self.cmd_args.seekpos}')
+        self.logger.info(f'finished: {tstamp} | {self.args.seekpos}')
         runtime = stop_time - start_time
         self.logger.info('runtime: %6.3f' %runtime)
         self.end()
@@ -309,24 +325,7 @@ class BaseMapper(object):
             return
         tmap_parser = AllMappingsParser(tmap_json)
         for hugo in tmap_parser.get_genes():
-            sos = tmap_parser.get_uniq_sos_for_gene(genes=[hugo])
-            for so in gene_level_so_exclude:
-                if so in sos:
-                    sos.remove(so)
-            if len(sos) == 0:
-                tmap_parser.delete_gene(hugo)
-                continue
-            so = most_severe_so(tmap_parser.get_uniq_sos_for_gene(genes=[hugo]))
-            try:
-                self.gene_info[hugo]['variant_count'] += 1
-            except KeyError:
-                self.gene_info[hugo] = {'hugo': hugo,
-                                        'variant_count': 1,
-                                        'so_counts': {}}
-            try:
-                self.gene_info[hugo]['so_counts'][so] += 1
-            except KeyError:
-                self.gene_info[hugo]['so_counts'][so] = 1
+            self.gene_info[hugo] = True
 
     def _write_crg(self):
         """
@@ -338,20 +337,6 @@ class BaseMapper(object):
             gene = self.gene_info[hugo]
             crg_data = {x['name']:'' for x in crg_def}
             crg_data['hugo'] = hugo
-            crg_data['num_variants'] = gene['variant_count']
-            so_count_toks = []
-            worst_so = ''
-            try:
-                worst_so = most_severe_so((gene['so_counts'].keys()))
-            except:
-                pass
-            sorted_counts = list(gene['so_counts'].items())
-            # Sort by SO occurence, descending
-            sorted_counts.sort(key=lambda l: so_severity.index(l[0]), reverse=True)
-            for so, so_count in sorted_counts:
-                so_count_toks.append('%s(%d)' %(so, so_count))
-            crg_data['so'] = worst_so
-            crg_data['all_so'] = ','.join(so_count_toks)
             self.crg_writer.write_data(crg_data)
 
     def _log_runtime_error(self, ln, line, e):
@@ -360,19 +345,21 @@ class BaseMapper(object):
             self.unique_excs.append(err_str)
             self.logger.error(err_str)
         self.error_logger.error('\nLINE:{:d}\nINPUT:{}\nERROR:{}\n#'.format(ln, line[:-1], str(e)))
-        if not(isinstance(e, InvalidData)):
-            raise e
+        # if not(isinstance(e, InvalidData)):
+        #     raise e
 
     async def get_gene_summary_data (self, cf):
         #print('            {}: started getting gene summary data'.format(self.module_name))
         t = time.time()
-        hugos = await cf.get_filtered_hugo_list()
+        hugos = await cf.exec_db(cf.get_filtered_hugo_list)
+        # Below is to fix opening oc 1.8.0 jobs with oc 1.8.1.
+        # TODO: Remove it after a while and add 1.8.0 to the db update chain in cravat_util.
         cols = ['base__' + coldef['name'] \
-                for coldef in crx_def]
+                for coldef in crx_def if coldef['name'] != 'cchange']
         cols.extend(['tagsampler__numsample'])
         data = {}
         t = time.time()
-        rows = await cf.get_variant_data_for_cols(cols)
+        rows = await cf.exec_db(cf.get_variant_data_for_cols, cols)
         rows_by_hugo = {}
         t = time.time()
         for row in rows:
