@@ -53,9 +53,6 @@ if not jobs_dir_key in conf:
     if os.path.exists(default_jobs_dir) == False:
         os.mkdir(default_jobs_dir)
     conf[jobs_dir_key] = default_jobs_dir
-    wf = open(system_conf_path, 'w')
-    yaml.dump(conf, wf, default_flow_style=False)
-    wf.close()
 # log dir
 log_dir_key = 'log_dir'
 log_dir_name = 'logs'
@@ -64,9 +61,6 @@ if not log_dir_key in conf:
     if os.path.exists(default_log_dir) == False:
         os.mkdir(default_log_dir)
     conf[log_dir_key] = default_log_dir
-    wf = open(system_conf_path, 'w')
-    yaml.dump(conf, wf, default_flow_style=False)
-    wf.close()
 # Live conf
 live_conf_fname = 'live.yml'
 
@@ -95,8 +89,8 @@ crm_def = [{'name':'original_line', 'title':'Original Line', 'type':'int', 'widt
     ]
 crm_idx = [['uid'],['tags']]
 crs_def = [{'name':'uid', 'title':'UID', 'type':'int', 'width': 70},
-           {'name':'sample_id', 'title':'Sample', 'type':'string', 'width': 90, 'category': 'multi'}]
-crs_idx = [['uid'], ['sample_id']]
+           {'name':'sample_id', 'title':'Sample', 'type':'string', 'width': 90}]
+crs_idx = [['uid'], ['sample_id'],['sample_id','uid']]
 crv_def = [{'name':'uid', 'title':'UID', 'type':'int', 'width': 60, 'hidden':True, 'filterable': False},
            {'name':'chrom', 'title':'Chrom', 'type':'string', 'width': 50, 'category': 'single', 'filterable': True},
            {'name':'pos', 'title':'Position', 'type':'int', 'width': 80, 'filterable': True},
@@ -108,32 +102,15 @@ crv_idx = [['uid']]
 crx_def = crv_def + \
           [{'name':'coding', 'title':'Coding', 'type':'string', 'width': 50, 'category': 'single',
                'categories': ['Y']},
-           {'name':'hugo', 'title':'Hugo', 'type':'string', 'width': 70, 'filterable': True},
-           {'name':'transcript', 'title':'Transcript', 'type':'string', 'width': 135, 'hidden':True, 'filterable': False},
-           {'name':'so', 'title':'Sequence Ontology', 'type':'string', 'width': 120, 'category': 'single',
-               'categories': [
-                   '2KD',
-                   '2KU', 
-                   'UT3', 
-                   'UT5', 
-                   'INT', 
-                   'UNK', 
-                   'SYN', 
-                   'MIS', 
-                   'CSS', 
-                   'IND', 
-                   'INI', 
-                   'STL', 
-                   'SPL', 
-                   'STG', 
-                   'FSD', 
-                   'FSI'], 'filterable': True},
-           #{'name':'cchange', 'title': 'cDNA change', 'type': 'string', 'width': 70, 'filterable': False},
+           {'name':'hugo', 'title':'Gene', 'type':'string', 'width': 70, 'filterable': True},
+           {'name':'transcript', 'title':'Transcript', 'type':'string', 'width': 135, 'hidden':False, 'filterable': False},
+           {'name':'so', 'title':'Sequence Ontology', 'type':'string', 'width': 120, 'category': 'single', 'filterable': True},
+           {'name':'cchange', 'title': 'cDNA change', 'type': 'string', 'width': 70, 'filterable': False},
            {'name':'achange', 'title':'Protein Change', 'type':'string', 'width': 55, 'filterable': False},
            {'name':'all_mappings', 'title':'All Mappings', 'type':'string', 'width': 100, 'hidden':True, 'filterable': False},
            ]
 crx_idx = [['uid']]
-crg_def = [{'name':'hugo', 'title':'Hugo', 'type':'string', 'width': 70, 'filterable': True},
+crg_def = [{'name':'hugo', 'title':'Gene', 'type':'string', 'width': 70, 'filterable': True},
            {'name': 'note', 'title': 'Note', 'type': 'string', 'width': 50},
           ]
 crg_idx = [['hugo']]
@@ -159,8 +136,6 @@ VARIANT = 0
 GENE = 1
 LEVELS = {'variant': VARIANT, 'gene': GENE}
 
-viewer_effective_digits = 3
-
 gene_level_so_exclude = ['2KU', '2KD']
 
 base_smartfilters = [
@@ -176,6 +151,20 @@ base_smartfilters = [
         'filter': {
             'operator': 'and',
             'rules': [
+                {
+                    'operator': 'or',
+                    'rules': [
+                        {
+                            'column': 'gnomad3__af', 
+                            'test': 'lessThanEq',
+                            'value': '${value}'
+                        },
+                        {
+                            'column': 'gnomad3__af', 
+                            'test': 'noData'
+                        },
+                    ]
+                },
                 {
                     'operator': 'or',
                     'rules': [
@@ -187,7 +176,7 @@ base_smartfilters = [
                         {
                             'column': 'gnomad__af', 
                             'test': 'noData'
-                        }
+                        },
                     ]
                 },
                 {
@@ -287,3 +276,5 @@ default_assembly = 'hg38'
 default_assembly_key = 'default_assembly'
 assembly_choices = ['hg38', 'hg19', 'hg18']
 publish_time_fmt = '%Y-%m-%dT%H:%M:%S.%f%z'
+
+install_tempdir_name = 'temp'
\ No newline at end of file
