@@ -17,6 +17,10 @@ import json
 import gzip
 from collections import defaultdict
 from cravat.base_converter import BaseConverter
+import cravat
+import re
+
+STDIN = 'stdin'
 
 class VTracker:
     """ This helper class is used to identify the unique variants from the input 
@@ -80,10 +84,12 @@ class MasterCravatConverter(object):
         self.cmd_args = None
         self.output_dir = None
         self.output_base_fname = None
+        self.pipeinput = False
         self.chromdict = {'chrx': 'chrX', 'chry': 'chrY', 'chrMT': 'chrM', 'chrMt': 'chrM', 'chr23': 'chrX', 'chr24': 'chrY'}
         self._parse_cmd_args(args)
         self._setup_logger()
         self.vtracker = VTracker(deduplicate=not(self.unique_variants))
+        self.wgsreader = cravat.get_wgs_reader(assembly='hg38')
 
     def _parse_cmd_args(self, args):
         """ Parse the arguments in sys.argv """
@@ -91,10 +97,12 @@ class MasterCravatConverter(object):
         parser.add_argument('path',
             help='Path to this converter\'s python module')
         parser.add_argument('inputs',
-            nargs='+',
+            nargs='*',
+            default=None,
             help='Files to be converted to .crv')
         parser.add_argument('-f',
             dest='format',
+            default=None,
             help='Specify an input format')
         parser.add_argument('-n', '--name',
             dest='name',
@@ -117,15 +125,26 @@ class MasterCravatConverter(object):
             action='store_true',
             help=argparse.SUPPRESS)
         parsed_args = parser.parse_args(args)
-        self.input_paths = [os.path.abspath(x) for x in parsed_args.inputs]
-        self.input_path_dict = {}
-        self.input_path_dict2 = {}
-        for i in range(len(self.input_paths)):
-            self.input_path_dict[i] = self.input_paths[i]
-            self.input_path_dict2[self.input_paths[i]] = i
         if parsed_args.format:
             self.input_format = parsed_args.format
+        if parsed_args.inputs is None:
+            raise ExpectedException('Input files are not given.')
+        if parsed_args.inputs is not None and len(parsed_args.inputs) == 1 and parsed_args.inputs[0] == '-':
+            self.pipeinput = True
+        if self.pipeinput == False:
+            self.input_paths = [os.path.abspath(x) for x in parsed_args.inputs if x != '-']
+        else:
+            self.input_paths = [f'./{STDIN}']
         self.input_dir = os.path.dirname(self.input_paths[0])
+        self.input_path_dict = {}
+        self.input_path_dict2 = {}
+        if self.pipeinput == False:
+            for i in range(len(self.input_paths)):
+                self.input_path_dict[i] = self.input_paths[i]
+                self.input_path_dict2[self.input_paths[i]] = i
+        else:
+            self.input_path_dict[0] = self.input_paths[0]
+            self.input_path_dict2[STDIN] = 0
         if parsed_args.output_dir:
             self.output_dir = parsed_args.output_dir
         else:
@@ -153,13 +172,16 @@ class MasterCravatConverter(object):
         """ Do necesarry pre-run tasks """
         if self.ready_to_convert: return
         # Open file handle to input path
-        for input_path in self.input_paths:
-            encoding = detect_encoding(input_path)
-            if input_path.endswith('.gz'):
-                f = gzip.open(input_path, mode='rt', encoding=encoding)
-            else:
-                f = open(input_path, encoding=encoding)
-            self.input_files.append(f)
+        if self.pipeinput == False:
+            for input_path in self.input_paths:
+                encoding = detect_encoding(input_path)
+                if input_path.endswith('.gz'):
+                    f = gzip.open(input_path, mode='rt', encoding=encoding)
+                else:
+                    f = open(input_path, encoding=encoding)
+                self.input_files.append(f)
+        else:
+            self.input_files = [sys.stdin]
         # Read in the available converters
         self._initialize_converters()
         # Select the converter that matches the input format
@@ -173,7 +195,10 @@ class MasterCravatConverter(object):
         """ Open a log file and set up log handler """
         self.logger = logging.getLogger('cravat.converter')
         self.logger.info('started: %s' %time.asctime())
-        self.logger.info('input files: %s' %', '.join(self.input_paths))
+        if self.pipeinput == False:
+            self.logger.info('Input file(s): %s' %', '.join(self.input_paths))
+        else:
+            self.logger.info(f'Input file(s): {STDIN}')
         if self.do_liftover:
             self.logger.info('liftover from %s' %self.input_assembly)
         self.error_logger = logging.getLogger('error.converter')
@@ -186,7 +211,7 @@ class MasterCravatConverter(object):
             python modules. Initializes the CravatConverter class from that
             module and places them in a dict keyed by their input format
         """
-        for module_info in au.get_local_module_infos_of_type('converter', update=False).values():
+        for module_info in au.get_local_module_infos_of_type('converter').values():
             # path based import from https://docs.python.org/3/library/importlib.html#importing-a-source-file-directly
             spec = importlib.util.spec_from_file_location(module_info.name,
                                                           module_info.script_path)
@@ -213,32 +238,39 @@ class MasterCravatConverter(object):
                 raise ExpectedException ('Invalid input format. Please select from [%s]' \
                          %', '.join(self.possible_formats))
         else:
-            valid_formats = []
-            first_file = self.input_files[0]
-            first_file.seek(0)
-            for converter_name, converter in self.converters.items():
-                check_success = converter.check_format(first_file)
+            if self.pipeinput == False:
+                valid_formats = []
+                first_file = self.input_files[0]
                 first_file.seek(0)
-                if check_success: valid_formats.append(converter_name)
-            if len(valid_formats) == 0:
-                fn = os.path.basename(first_file.name)
-                msg = f'Input format could not be determined for file {fn}. Additional input format converters are available. View available converters in the store or with "oc module ls -a -t converter"'
-                raise ExpectedException(msg)
-            elif len(valid_formats) > 1:
-                raise ExpectedException('Input format ambiguous in [%s]. '\
-                            %', '.join(valid_formats)\
-                         +'Please specify an input format.')
+                for converter_name, converter in self.converters.items():
+                    check_success = converter.check_format(first_file)
+                    first_file.seek(0)
+                    if check_success: valid_formats.append(converter_name)
+                if len(valid_formats) == 0:
+                    fn = os.path.basename(first_file.name)
+                    msg = f'Input format could not be determined for file {fn}. Additional input format converters are available. View available converters in the store or with "oc module ls -a -t converter"'
+                    raise ExpectedException(msg)
+                elif len(valid_formats) > 1:
+                    raise ExpectedException('Input format ambiguous in [%s]. '\
+                                %', '.join(valid_formats)\
+                             +'Please specify an input format.')
+                else:
+                    self.input_format = valid_formats[0]
             else:
-                self.input_format = valid_formats[0]
+                if self.input_format is None:
+                    msg = 'Input should be specified with --input-format option when input is given through pipe'
+                    print(f'\n{msg}')
+                    raise ExpectedException(msg)
         self.primary_converter = self.converters[self.input_format]
         self.primary_converter.output_dir = self.output_dir
         self.primary_converter.run_name = self.output_base_fname
-        if len(self.input_files) > 1:
-            for f in self.input_files[1:]:
-                if not self.primary_converter.check_format(f):
-                    raise ExpectedException('Inconsistent file types')
-                else:
-                    f.seek(0)
+        if self.pipeinput == False:
+            if len(self.input_files) > 1:
+                for f in self.input_files[1:]:
+                    if not self.primary_converter.check_format(f):
+                        raise ExpectedException('Inconsistent file types')
+                    else:
+                        f.seek(0)
         self.logger.info('input format: %s' %self.input_format)
 
     def _open_output_files (self):
@@ -302,15 +334,24 @@ class MasterCravatConverter(object):
         multiple_files = len(self.input_files) > 1
         fileno = 0
         total_lnum = 0
+        base_re = re.compile('^[ATGC]+|[-]+$')
         for f in self.input_files:
+            if self.pipeinput == True:
+                fname = STDIN
+            else:
+                fname = f.name
             fileno += 1
             self.primary_converter.setup(f)
-            f.seek(0)
+            if self.pipeinput == False:
+                f.seek(0)
             read_lnum = 0
             write_lnum = 0
             num_errors = 0
-            for l in f:
+            if self.pipeinput:
+                cur_fname = STDIN
+            else:
                 cur_fname = os.path.basename(f.name)
+            for l in f:
                 samp_prefix = cur_fname
                 read_lnum += 1
                 try:
@@ -321,57 +362,69 @@ class MasterCravatConverter(object):
                     if all_wdicts is BaseConverter.IGNORE:
                         continue
                     total_lnum += 1
+                    if all_wdicts:
+                        UIDMap = []
+                        no_unique_var = 0
+                        for wdict_no in range(len(all_wdicts)):
+                            wdict = all_wdicts[wdict_no]
+                            chrom = wdict['chrom']
+                            pos = wdict['pos']
+                            if chrom is not None:
+                                if not chrom.startswith('chr'):
+                                    chrom = 'chr' + chrom
+                                wdict['chrom'] = self.chromdict.get(chrom, chrom)
+                                if multiple_files:
+                                    if wdict['sample_id']:
+                                        wdict['sample_id'] = '__'.join([samp_prefix, wdict['sample_id']])
+                                    else:
+                                        wdict['sample_id'] = samp_prefix
+                                if self.do_liftover:
+                                    prelift_wdict = copy.copy(wdict)
+                                    wdict['chrom'], wdict['pos'] = self.liftover(wdict['chrom'], wdict['pos'])
+                                if 'ref_base' not in wdict or wdict['ref_base'] == '':
+                                    wdict['ref_base'] = self.wgsreader.get_bases(chrom, int(wdict['pos']))
+                                else:
+                                    ref_base = wdict['ref_base']
+                                    if ref_base == '' and wdict['alt_base'] not in ['A','T','C','G']:
+                                        raise BadFormatError('Reference base required for non SNV')
+                                    elif ref_base is None or ref_base == '':
+                                        wdict['ref_base'] = self.wgsreader.get_bases(chrom, int(pos))
+                                if base_re.fullmatch(wdict['ref_base']) is None:
+                                    raise BadFormatError('Invalid reference base')
+                                if base_re.fullmatch(wdict['alt_base']) is None:
+                                    raise BadFormatError('Invalid alternate base')
+                                p, r, a = int(wdict['pos']), wdict['ref_base'], wdict['alt_base']
+                                new_pos, new_ref, new_alt = self.standardize_pos_ref_alt('+', p, r, a)
+                                unique, UID = self.vtracker.addVar(wdict['chrom'], new_pos, new_ref, new_alt)
+                                wdict['uid'] = UID
+                                if unique:
+                                    write_lnum += 1
+                                    self.crv_writer.write_data(wdict)
+                                    if self.do_liftover:
+                                        prelift_wdict['uid'] = UID
+                                        self.crl_writer.write_data(prelift_wdict)
+                                    # addl_operation errors shouldnt prevent variant from writing
+                                    try:
+                                        self.primary_converter.addl_operation_for_unique_variant(wdict, no_unique_var)
+                                    except Exception as e:
+                                        self._log_conversion_error(read_lnum, l, e)
+                                    no_unique_var += 1
+                                if UID not in UIDMap: 
+                                    #For this input line, only write to the .crm if the UID has not yet been written to the map file.   
+                                    self.crm_writer.write_data({
+                                        'original_line': read_lnum, 
+                                        'tags': wdict['tags'], 
+                                        'uid': UID, 
+                                        'fileno': self.input_path_dict2[fname]
+                                    })
+                                    UIDMap.append(UID)
+                            self.crs_writer.write_data(wdict)
+                    else:
+                        raise ExpectedException('No conversion result')
                 except Exception as e:
                     num_errors += 1
                     self._log_conversion_error(read_lnum, l, e)
                     continue
-                if all_wdicts:
-                    UIDMap = []
-                    no_unique_var = 0
-                    for wdict_no in range(len(all_wdicts)):
-                        wdict = all_wdicts[wdict_no]
-                        chrom = wdict['chrom']
-                        if chrom is not None:
-                            if not chrom.startswith('chr'): chrom = 'chr' + chrom
-                            wdict['chrom'] = self.chromdict.get(chrom, chrom)
-                            if multiple_files:
-                                if wdict['sample_id']:
-                                    wdict['sample_id'] = '__'.join([samp_prefix, wdict['sample_id']])
-                                else:
-                                    wdict['sample_id'] = samp_prefix
-                            if wdict['ref_base'] == '' and wdict['alt_base'] not in ['A','T','C','G']:
-                                num_errors += 1
-                                e = BadFormatError('Reference base required for non SNV')
-                                self._log_conversion_error(read_lnum, l, e)
-                                continue
-                            if self.do_liftover:
-                                prelift_wdict = copy.copy(wdict)
-                                try:
-                                    wdict['chrom'], wdict['pos'] = self.liftover(wdict['chrom'], wdict['pos'])
-                                except LiftoverFailure as e:
-                                    num_errors += 1
-                                    self._log_conversion_error(read_lnum, l, e)
-                                    continue
-                            p, r, a = int(wdict['pos']), wdict['ref_base'], wdict['alt_base']
-                            new_pos, new_ref, new_alt = self.standardize_pos_ref_alt('+', p, r, a)
-                            unique, UID = self.vtracker.addVar(wdict['chrom'], new_pos, new_ref, new_alt)
-                            wdict['uid'] = UID
-                            if unique:
-                                write_lnum += 1
-                                self.crv_writer.write_data(wdict)
-                                if self.do_liftover:
-                                    prelift_wdict['uid'] = UID
-                                    self.crl_writer.write_data(prelift_wdict)
-                                self.primary_converter.addl_operation_for_unique_variant(wdict, no_unique_var)
-                                no_unique_var += 1
-                            if UID not in UIDMap: 
-                                #For this input line, only write to the .crm if the UID has not yet been written to the map file.   
-                                self.crm_writer.write_data({'original_line': read_lnum, 'tags': wdict['tags'], 'uid': UID, 'fileno': self.input_path_dict2[f.name]})
-                                UIDMap.append(UID)
-                        self.crs_writer.write_data(wdict)
-                else:
-                    e = ExpectedException('No conversion result')
-                    self._log_conversion_error(read_lnum, l, e)
             cur_time = time.time()
             if total_lnum % 10000 == 0 or cur_time - last_status_update_time > 3:
                 self.status_writer.queue_status_update('status', 'Running {} ({}): line {}'.format('Converter', cur_fname, read_lnum))
