@@ -3,7 +3,7 @@ import argparse
 import os
 import sys
 import oyaml as yaml
-import aiosqlite3
+import aiosqlite
 import json
 import re
 import time
@@ -145,6 +145,7 @@ class CravatFilter ():
         else:
             self.stdout = False
         self.dbpath = dbpath
+        self.conn = None
         self.filterpath = None
         self.cmd = None
         self.level = None
@@ -152,8 +153,6 @@ class CravatFilter ():
         self.savefiltername = None
         self.filtername = None
         self.filterstring = None
-        self.conn = None
-        self.cursor = None
         if filter != None:
             self.filter = filter
         else:
@@ -165,12 +164,28 @@ class CravatFilter ():
                 self.filterpath = filterpath
         self.filtertable = 'filter'
         self.generows = {}
+        #self.open_conns = {}
+
+    #async def close_conns (self):
+    #    for conn in self.open_conns.values():
+    #        await conn.close()
+
+    async def exec_db (self, func, *args, **kwargs):
+        conn = await self.get_db_conn()
+        cursor = await conn.cursor()
+        #conn_hash = conn.__hash__()
+        #self.open_conns[conn_hash] = conn
+        ret = await func(*args, conn=conn, cursor=cursor, **kwargs)
+        #await cursor.close()
+        #await conn.close()
+        #del self.open_conns[conn_hash]
+        return ret
 
     async def second_init (self):
         if self.mode == 'sub':
             if self.dbpath != None:
                 await self.connect_db()
-            await self.loadfilter()
+            await self.exec_db(self.loadfilter)
 
     async def run (self, cmd=None, args=None, dbpath=None, filter=None):
         if args != None:
@@ -180,34 +195,29 @@ class CravatFilter ():
 
         if dbpath != None:
             self.dbpath = dbpath
-            await self.connect_db()
-        elif self.dbpath != None and self.cursor == None:
-            await self.connect_db()
 
         # Loads filter.
         if filter != None:
             self.filter = filter
         elif (self.filtername != None or self.filterpath != None or 
             self.filterstring != None) and self.filter == None:
-            await self.loadfilter()
+            await self.exec_db(self.loadfilter)
 
         ret = None
-        if self.cursor != None: # and self.filter != None:
-            #if self.cmd == 'uidpipe':
-            #    ret = await self.run_level_based_func(self.getuiditerator)
+        if self.dbpath is not None:
             if self.cmd == 'count':
-                ret = await self.run_level_based_func(self.getcount)
+                ret = await self.run_level_based_func(self.exec_db(self.getcount))
             elif self.cmd == 'rows':
-                ret = await self.run_level_based_func(self.getrows)
+                ret = await self.run_level_based_func(self.exec_db(self.getrows))
             elif self.cmd == 'pipe':
-                ret = await self.run_level_based_func(self.getiterator)
-        elif self.cursor != None and self.cmd == 'list':
-            ret = self.listfilter()
+                ret = await self.run_level_based_func(self.exec_db(self.getiterator))
+        elif self.dbpath is not None and self.cmd == 'list':
+            ret = self.exec_db(self.listfilter)
 
         # Saves filter.
         if self.filter != None:
             if self.cmd == 'save' or self.savefiltername != None:
-                ret = self.savefilter()
+                ret = self.exec_db(self.savefilter)
 
         return ret
 
@@ -262,48 +272,54 @@ class CravatFilter ():
         self.filtername = parsed_args.filtername
         self.filterstring = parsed_args.filterstring
 
+    async def get_db_conn (self):
+        if self.dbpath is None:
+            return None
+        if self.conn is None:
+            self.conn = await aiosqlite.connect(self.dbpath)
+        return self.conn
+
     async def connect_db (self, dbpath=None):
         if dbpath != None:
             self.dbpath = dbpath
-        self.conn = await aiosqlite3.connect(self.dbpath)
-        self.cursor = await self.conn.cursor()
-        self.conn.create_function('regexp', 2, regexp)
-        await self.set_aliases()
+        conn = await self.get_db_conn()
+        await conn.create_function('regexp', 2, regexp)
+        #await conn.close()
+        await self.exec_db(self.set_aliases)
 
-    async def set_aliases (self):
+    async def set_aliases (self, conn=None, cursor=None):
         self.table_aliases = {'variant':'t','gene':'g'}
         self.column_prefixes = {}
         q = 'pragma table_info(variant)'
-        await self.cursor.execute(q)
-        self.column_prefixes.update({row[1]:self.table_aliases['variant'] for row in await self.cursor.fetchall()})
+        await cursor.execute(q)
+        self.column_prefixes.update({row[1]:self.table_aliases['variant'] for row in await cursor.fetchall()})
         q = 'pragma table_info(gene)'
-        await self.cursor.execute(q)
-        self.column_prefixes.update({row[1]:self.table_aliases['gene'] for row in await self.cursor.fetchall()})
+        await cursor.execute(q)
+        self.column_prefixes.update({row[1]:self.table_aliases['gene'] for row in await cursor.fetchall()})
 
     async def close_db (self):
-        await self.cursor.close()
-        self.conn.close()
+        if self.conn is not None:
+            await self.conn.close()
 
-    async def create_filtertable (self):
-        if self.cursor == None:
+    async def create_filtertable (self, conn=None, cursor=None):
+        if conn is None:
             return
         sql = 'create table ' + self.filtertable + ' (name text, criteria text)'
-        await self.cursor.execute(sql)
-        self.conn.commit()
+        await cursor.execute(sql)
+        await conn.commit()
 
-    async def filtertable_exists (self):
+    async def filtertable_exists (self, conn=None, cursor=None):
         sql = 'select name from sqlite_master where ' +\
             'type="table" and name="' + self.filtertable + '"'
-        await self.cursor.execute(sql)
-        for row in await self.cursor.fetchone():
-            if row == None:
-                ret = False
-            else:
-                ret = True
+        await cursor.execute(sql)
+        row = await cursor.fetchone()
+        if row == None:
+            ret = False
+        else:
+            ret = True
         return ret
 
-    async def loadfilter (self, filterpath=None, filtername=None, 
-            filterstring=None, filter=None):
+    async def loadfilter (self, filterpath=None, filtername=None, filterstring=None, filter=None, conn=None, cursor=None):
         if filterpath != None:
             self.filterpath = filterpath
         if filtername != None:
@@ -312,15 +328,16 @@ class CravatFilter ():
             self.filterstring = filterstring
         if filter != None:
             self.filter = filter
+        filter_table_present = await self.exec_db(self.filtertable_exists)
         if self.filter:
             pass
         elif self.filterstring is not None:
             self.filterstring = self.filterstring.replace("'", '"')
             self.filter = json.loads(self.filterstring)
-        elif self.filtername is not None and self.filtertable_exists():
-            await self.cursor.execute('select criteria from ' + self.filtertable +
+        elif self.filtername is not None and filter_table_present:
+            await cursor.execute('select criteria from ' + self.filtertable +
                 ' where name="' + self.filtername + '"')
-            criteria = await self.cursor.fetchone()
+            criteria = await cursor.fetchone()
             if criteria != None:
                 self.filter = json.loads(criteria[0])
         elif self.filterpath is not None and os.path.exists(self.filterpath):
@@ -333,14 +350,14 @@ class CravatFilter ():
         if self.filter is None:
             self.filter = {}
 
-    async def delete_filtered_uid_table (self):
-        await self.cursor.execute('pragma synchronous=0')
+    async def delete_filtered_uid_table (self, conn=None, cursor=None):
+        await cursor.execute('pragma synchronous=0')
         q = 'drop table if exists variant_filtered'
-        await self.cursor.execute(q)
+        await cursor.execute(q)
         q = 'drop table if exists gene_filtered'
-        await self.cursor.execute(q)
-        self.conn.commit()
-        await self.cursor.execute('pragma synchronous=2')
+        await cursor.execute(q)
+        await conn.commit()
+        await cursor.execute('pragma synchronous=2')
 
     def getwhere (self, level):
         where = ''
@@ -354,22 +371,26 @@ class CravatFilter ():
         return where
 
     def getvariantcount (self):
-        return self.getcount('variant')
+        loop = asyncio.get_event_loop()
+        count = loop.run_until_complete(self.exec_db(self.getcount, 'variant'))
+        return count
 
     def getgenecount (self):
-        return self.getcount('gene')
+        loop = asyncio.get_event_loop()
+        count = loop.run_until_complete(self.exec_db(self.getcount, 'gene'))
+        return count
 
-    async def getcount (self, level='variant'):
+    async def getcount (self, level='variant', conn=None, cursor=None):
         bypassfilter = (self.filter == {})
         level = 'variant'
-        await self.make_filtered_uid_table()
+        await self.exec_db(self.make_filtered_uid_table)
         if bypassfilter:
             ftable = level
         else:
             ftable = level + '_filtered'
         q = 'select count(*) from ' + ftable
-        await self.cursor.execute(q)
-        for row in await self.cursor.fetchone():
+        await cursor.execute(q)
+        for row in await cursor.fetchone():
             n = row
         if self.stdout == True:
             print('#' + level)
@@ -377,36 +398,40 @@ class CravatFilter ():
         return n
 
     def getvariantrows (self):
-        return self.getrows('variant')
+        loop = asyncio.get_event_loop()
+        rows = loop.run_until_complete(self.exec_db(self.getrows, 'variant'))
+        return rows
 
     def getgenerows (self):
-        return self.getrows('gene')
+        loop = asyncio.get_event_loop()
+        rows = loop.run_until_complete(self.exec_db(self.getrows, 'gene'))
+        return rows
 
-    async def getrows (self, level='variant'):
+    async def getrows (self, level='variant', conn=None, cursor=None):
         if level != 'variant':
             return
         where = self.getwhere(level)
-        await self.make_filtered_sample_table()
-        await self.make_gene_list_table()
-        await self.conn.commit()
+        await self.exec_db(self.make_filtered_sample_table)
+        await self.exec_db(self.make_gene_list_table)
+        await conn.commit()
         q = f'select t.* from variant as t join fsample as s on t.base__uid=s.base__uid'
         if isinstance(self.filter,dict) and len(self.filter.get('genes',[])) > 0:
             q += ' join gene_list as gl on t.base__hugo=gl.base__hugo'
         if 'g.' in where:
             q += ' join gene as g on t.base__hugo=g.base__hugo'
-        await self.cursor.execute(q)
-        ret = [list(v) for v in await self.cursor.fetchall()]
+        await cursor.execute(q)
+        ret = [list(v) for v in await cursor.fetchall()]
         if self.stdout == True:
             print('#' + level)
             for row in ret:
                 print('\t'.join([str(v) for v in row]))
         return ret
 
-    async def make_generows (self):
+    async def make_generows (self, conn=None, cursor=None):
         t = time.time()
         q = 'select * from gene'
-        await self.cursor.execute(q)
-        rows = await self.cursor.fetchall()
+        await cursor.execute(q)
+        rows = await cursor.fetchall()
         self.generows = {}
         for row in rows:
             hugo = row[0]
@@ -416,24 +441,28 @@ class CravatFilter ():
         if hugo is None:
             return None
         if bool(self.generows) == False:
-            await self.make_generows()
+            await self.exec_db(self.make_generows)
         row = self.generows.get(hugo)
         return row
 
     def getvariantiterator (self):
-        return self.getiterator('variant')
+        loop = asyncio.get_event_loop()
+        iterator = loop.run_until_complete(self.exec_db(self.getiterator, 'variant'))
+        return iterator
 
     def getgeneiterator (self):
-        return self.getiterator('gene')
+        loop = asyncio.get_event_loop()
+        iterator = loop.run_until_complete(self.exec_db(self.getiterator, 'gene'))
+        return iterator
 
-    async def getiterator (self, level='variant'):
+    async def getiterator (self, level='variant', conn=None, cursor=None):
         (sample_needed, tag_needed, include_where, exclude_where) = self.getwhere(level)
         sql = 'select *  from ' + level + include_where + ' except select * from ' + level + exclude_where
-        await self.cursor.execute(sql)
-        it = await self.cursor.fetchall()
+        await cursor.execute(sql)
+        it = await cursor.fetchall()
         return it
 
-    async def get_filtered_iterator (self, level='variant'):
+    async def get_filtered_iterator (self, level='variant', conn=None, cursor=None):
         bypassfilter = (self.filter == {})
         if level == 'variant':
             kcol = 'base__uid'
@@ -463,22 +492,22 @@ class CravatFilter ():
         if level in ['variant', 'gene', 'sample', 'mapping']:
             if level == 'gene' and bypassfilter:
                 sql = 'pragma table_info(gene)'
-                await self.cursor.execute(sql)
-                rs = await self.cursor.fetchall()
+                await cursor.execute(sql)
+                rs = await cursor.fetchall()
                 colnames = ['gene.' + r[1] for r in rs if r[1] != 'base__hugo']
                 sql = 'select distinct variant.base__hugo, {} from variant inner join gene on variant.base__hugo==gene.base__hugo'.format(', '.join(colnames))
             else:
                 sql = 'select t.* from ' + table + ' as t'
                 if bypassfilter == False:
                     sql += ' inner join ' + ftable + ' as f on t.' + kcol + '=f.' + kcol
-        await self.cursor.execute(sql)
-        cols = [v[0] for v in self.cursor.description]
-        rows = await self.cursor.fetchall()
+        await cursor.execute(sql)
+        cols = [v[0] for v in cursor.description]
+        rows = await cursor.fetchall()
         return cols, rows
 
-    async def make_filtered_sample_table (self):
+    async def make_filtered_sample_table (self, conn=None, cursor=None):
         q = 'drop table if exists fsample'
-        await self.cursor.execute(q)
+        await cursor.execute(q)
         if 'sample' in self.filter:
             if 'require' in self.filter['sample']:
                 req = self.filter['sample']['require']
@@ -496,25 +525,22 @@ class CravatFilter ():
             q += ' where base__sample_id in ({})'.format(
                 ', '.join(['"{}"'.format(sid) for sid in req])
             )
-        # for s in req:
-        #     q += ' union select base__uid from sample where base__sample_id="{}"'.format(s)
         for s in rej:
            q += ' except select base__uid from sample where base__sample_id="{}"'.format(s)
-        await self.cursor.execute(q)
-        await self.conn.commit()
+        await cursor.execute(q)
+        await conn.commit()
 
-    async def make_filtered_uid_table (self):
+    async def make_filtered_uid_table (self, conn=None, cursor=None):
         t = time.time()
         bypassfilter = (self.filter == {})
         if bypassfilter == False:
-            await self.make_filtered_sample_table()
-            await self.make_gene_list_table()
-            await self.conn.commit()
+            await self.exec_db(self.make_filtered_sample_table)
+            await self.exec_db(self.make_gene_list_table)
             level = 'variant'
             vtable = level
             vftable = level + '_filtered'
             q = 'drop table if exists ' + vftable
-            await self.cursor.execute(q)
+            await cursor.execute(q)
             where = self.getwhere(level)
             q = 'create table {} as select t.base__uid from {} as t'.format(vftable, level)
             q += ' join fsample as s on t.base__uid=s.base__uid'
@@ -523,89 +549,84 @@ class CravatFilter ():
             if 'g.' in where:
                 q += ' join gene as g on t.base__hugo=g.base__hugo'
             q += ' '+where
-            await self.cursor.execute(q)
-            self.conn.commit()
+            await cursor.execute(q)
+            await conn.commit()
             t = time.time() - t
-    
-    async def make_gene_list_table (self):
+
+    async def make_gene_list_table (self, conn=None, cursor=None):
         tname = 'gene_list'
         q = 'drop table if exists {}'.format(tname)
-        await self.cursor.execute(q)
+        await cursor.execute(q)
         q = 'create table {} (base__hugo text)'.format(tname)
-        await self.cursor.execute(q)
+        await cursor.execute(q)
         if isinstance(self.filter,dict) and 'genes' in self.filter:
             tdata = [(hugo,) for hugo in self.filter['genes']]
         else:
             tdata = []
         if tdata:
             q = 'insert into {} (base__hugo) values (?)'.format(tname)
-            await self.cursor.executemany(q, tdata)
+            await cursor.executemany(q, tdata)
         else:
             q = 'insert into {} select base__hugo from gene'.format(tname)
-            await self.cursor.execute(q)
-        await self.conn.commit()
-    
-    async def make_filtered_hugo_table (self):
+            await cursor.execute(q)
+        await conn.commit()
+
+    async def make_filtered_hugo_table (self, conn=None, cursor=None):
         bypassfilter = (self.filter == {})
         if bypassfilter == False:
-            await self.cursor.execute('pragma synchronous=0')
+            await cursor.execute('pragma synchronous=0')
             level = 'gene'
             vtable = 'variant'
             vftable = vtable + '_filtered'
             gftable = level + '_filtered'
             q = 'drop table if exists ' + gftable
-            await self.cursor.execute(q)
+            await cursor.execute(q)
             q = 'create table ' + gftable +\
                 ' as select distinct v.base__hugo from ' + vtable + ' as v'\
                 ' inner join ' + vftable + ' as vf on vf.base__uid=v.base__uid'\
                 ' where v.base__hugo is not null'
-            await self.cursor.execute(q)
-            await self.cursor.execute('pragma synchronous=2')
+            await cursor.execute(q)
+            await cursor.execute('pragma synchronous=2')
 
-    async def savefilter (self, name=None):
-        if self.cursor == None or self.filter == None:
+    async def savefilter (self, name=None, conn=None, cursor=None):
+        if conn is None or self.filter is None:
             return
-
         if name == None:
             if self.savefiltername != None:
                 name = self.savefiltername
             else:
                 name = 'default'
-
         # Creates filter save table if not exists.
-        await self.cursor.execute('select name from sqlite_master where ' +
+        await cursor.execute('select name from sqlite_master where ' +
             'type="table" and name="' + self.filtertable + '"')
-        for ret in await self.cursor.fetchone():
+        for ret in await cursor.fetchone():
             if ret == None:
-                await self.cursor.execute('create table ' + self.filtertable +
+                await cursor.execute('create table ' + self.filtertable +
                     ' (name text unique, criteria text)')
-
         # Saves the filter.
         filterstr = json.dumps(self.filter)
         sql = 'insert or replace into ' + self.filtertable +\
             ' values ("' + name + '", \'' + filterstr + '\')'
-        await self.cursor.execute(sql)
-        self.conn.commit()
+        await cursor.execute(sql)
+        await conn.commit()
 
-    async def listfilter (self, name=None):
+    async def listfilter (self, name=None, conn=None, cursor=None):
         if name == None:
             if self.savefiltername != None:
                 name = self.savefiltername
             else:
                 name = 'default'
-
         # Creates filter save table if not exists.
-        await self.cursor.execute('select name from sqlite_master where ' +
+        await cursor.execute('select name from sqlite_master where ' +
             'type="table" and name="' + self.filtertable + '"')
-        for ret in await self.cursor.fetchone():
+        for ret in await cursor.fetchone():
             if ret == None:
-                await self.cursor.execute('create table ' + self.filtertable +
+                await cursor.execute('create table ' + self.filtertable +
                     ' (name text, criteria text)')
-
         sql = 'select name, criteria from ' + self.filtertable
-        await self.cursor.execute(sql)
+        await cursor.execute(sql)
         ret = {}
-        for row in await self.cursor.fetchall():
+        for row in await cursor.fetchall():
             name = row[0]
             criteria = json.loads(row[1])
             ret[name] = criteria
@@ -616,7 +637,6 @@ class CravatFilter ():
                     for column in criteria[level]:
                         print('        ' + column + ': ' + 
                               criteria[level][column])
-
         return ret
 
     def addvariantfilter (self, column, condition):
@@ -644,43 +664,43 @@ class CravatFilter ():
         if level in self.filter and column in self.filter[level]:
             del self.filter[level][column]
 
-    async def table_exists (self, table):
+    async def table_exists (self, table, conn=None, cursor=None):
         sql = 'select name from sqlite_master where type="table" and ' +\
             'name="' + table + '"'
-        await self.cursor.execute(sql)
-        for row in await self.cursor.fetchone():
+        await cursor.execute(sql)
+        for row in await cursor.fetchone():
             if row == None:
                 return False
             else:
                 return True
 
-    async def get_variant_iterator_filtered_uids_cols (self, cols):
+    async def get_variant_iterator_filtered_uids_cols (self, cols, conn=None, cursor=None):
         if cols[0] == 'base__uid':
             cols[0] = 'v.' + cols[0]
         bypassfilter = (self.filter == {})
         q = 'select ' + ','.join(cols) + ' from variant as v'
         if bypassfilter == False:
             q += ' inner join variant_filtered as f on v.base__uid=f.base__uid'
-        await self.cursor.execute(q)
-        rows = await self.cursor.fetchall()
+        await cursor.execute(q)
+        rows = await cursor.fetchall()
         for row in rows:
             d = {}
             for i in range(len(row)):
                 d[cols[i].split('__')[1]] = row[i]
             yield d
 
-    async def get_filtered_hugo_list (self):
+    async def get_filtered_hugo_list (self, conn=None, cursor=None):
         bypassfilter = (self.filter == {})
         if bypassfilter:
             q = 'select distinct variant.base__hugo from gene, variant where gene.base__hugo==variant.base__hugo'
         else:
             q = 'select base__hugo from gene_filtered'
-        await self.cursor.execute(q)
-        rows = await self.cursor.fetchall()
+        await cursor.execute(q)
+        rows = await cursor.fetchall()
         hugos = [row[0] for row in rows]
         return hugos
 
-    async def get_variant_data_for_cols (self, cols):
+    async def get_variant_data_for_cols (self, cols, conn=None, cursor=None):
         bypassfilter = (self.filter == {})
         if cols[0] == 'base__uid':
             cols[0] = 'v.base__uid'
@@ -689,11 +709,11 @@ class CravatFilter ():
             q += ' inner join variant_filtered as f on v.base__uid=f.base__uid'
         if cols[0] == 'v.base__uid':
             cols[0] = 'base__uid'
-        await self.cursor.execute(q)
-        rows = await self.cursor.fetchall()
+        await cursor.execute(q)
+        rows = await cursor.fetchall()
         return rows
 
-    async def get_variant_data_for_hugo (self, hugo, cols):
+    async def get_variant_data_for_hugo (self, hugo, cols, conn=None, cursor=None):
         bypassfilter = (self.filter == {})
         if cols[0] == 'base__uid':
             cols[0] = 'v.base__uid'
@@ -702,16 +722,16 @@ class CravatFilter ():
             q += ' inner join variant_filtered as f on v.base__uid=f.base__uid and v.base__hugo="{}"'.format(hugo)
         if cols[0] == 'v.base__uid':
             cols[0] = 'base__uid'
-        await self.cursor.execute(q)
-        rows = await self.cursor.fetchall()
+        await cursor.execute(q)
+        rows = await cursor.fetchall()
         return rows
 
-    async def get_result_levels (self):
+    async def get_result_levels (self, conn=None, cursor=None):
         q = 'select name from sqlite_master where type="table" and ' +\
             'name like "%_header"'
-        await self.cursor.execute(q)
+        await cursor.execute(q)
         table_names = []
-        for row in await self.cursor.fetchall():
+        for row in await cursor.fetchall():
             table_names.append(row[0].replace('_header', ''))
         return table_names
 
@@ -724,3 +744,4 @@ def main ():
     loop = asyncio.new_event_loop()
     cv = loop.run_until_complete(CravatFilter.create(mode='main'))
     loop.run_until_complete(cv.run(args=sys.argv[1:]))
+    loop.close()
