@@ -78,6 +78,8 @@ class CravatReader (CravatFile):
     def override_column(self, index, name, title=None, data_type='string', cats=[], category=None):
         if title == None:
             title = ' '.join(x.title() for x in name.split('_'))
+        if index not in self.columns:
+            self.columns[index] = ColumnDefinition({})
         self.columns[index].title= title
         self.columns[index].name = name
         self.columns[index].type = data_type
@@ -386,7 +388,9 @@ class AllMappingsParser (object):
     def get_uniq_sos (self):
         sos = {}
         for mapping in self.mappings:
-            sos[mapping.so] = True
+            #sos[mapping.so] = True
+            for so in mapping.so.split(','):
+                sos[so] = True
         sos = list(sos.keys())
         return sos
 
@@ -394,7 +398,9 @@ class AllMappingsParser (object):
         sos = {}
         for mapping in self.mappings:
             if mapping.gene in genes:
-                sos[mapping.so] = True
+                #sos[mapping.so] = True
+                for so in mapping.so.split(','):
+                    sos[so] = True
         sos = list(sos.keys())
         return sos
 
@@ -546,6 +552,7 @@ class ColumnDefinition (object):
             'col_filterable': self.filterable,
             'link_format': self.link_format,
             'col_genesummary': self.genesummary,
+            'col_index': self.index
         }
     
     def __iter__(self): # Allows casting to dict
