@@ -432,6 +432,7 @@ function getMostDownloadedModuleNames () {
 }
 
 function updateModuleGroupInfo () {
+    moduleGroupMembers = {};
     for (var mn in remoteModuleInfo) {
         var groups = remoteModuleInfo[mn]['groups'];
         if (groups != undefined && groups.length > 0) {
@@ -479,6 +480,9 @@ function updateModuleGroupInfo () {
         for (var j = 0; j < mns.length; j++) {
             var mn = mns[j];
             var m = remoteModuleInfo[mn];
+            if (m===undefined) { // Work if no internet
+                continue;
+            }
             var d1 = new Date(group.publish_time);
             var d2 = new Date(m.publish_time);
             if (d1 < d2) {
@@ -2153,7 +2157,7 @@ function connectWebSocket () {
             sdiv.style.color = 'black';
             sdiv.textContent = msg;
         }
-        if (msg.startsWith('Finished installation of')) {
+        if (msg.search('Finished installation of')>0) {
             delete installInfo[module];
             //installQueue = installQueue.filter(e => e != module);
             unqueue(module);
@@ -2163,11 +2167,11 @@ function connectWebSocket () {
                 var module = installQueue.shift();
                 installInfo[module] = {'msg': 'installing'};
             }
-        } else if (msg.startsWith('Aborted ')) {
+        } else if (msg.search('Aborted')>0) {
             delete installInfo[module];
             unqueue(module);
             setModuleTileInstallButton (module);
-        } else if (msg.startsWith('Unqueued')) {
+        } else if (msg.search('Unqueued')>0) {
             delete installInfo[module];
             unqueue(module);
             setModuleTileInstallButton (module);
