@@ -315,6 +315,8 @@ function populateJobTr (job) {
         addEl(td, getTn(job.username));
         addEl(jobTr, td);
     }
+    // Job ID
+    addEl(jobTr, addEl(getEl('td'), getTn(job.id)));
     // Input file name
     if (Array.isArray(job.orig_input_fname)) {
         input_fname = job.orig_input_fname.join(', ');
@@ -327,6 +329,15 @@ function populateJobTr (job) {
         input_fname_display = input_fname.substring(0, input_fname_display_limit) + '...';
     }
     addEl(jobTr, addEl(getEl('td'), getTn(input_fname_display)));
+    // Number of unique variants
+    var td = getEl('td');
+    td.style.textAlign = 'center';
+    var num = '';
+    if (job.num_unique_var != undefined) {
+        num = '' + job.num_unique_var;
+    }
+    td.textContent = num;
+    addEl(jobTr, td);
     // Number of annotators
     var annots = job.annotators;
     if (annots == undefined) {
@@ -357,16 +368,39 @@ function populateJobTr (job) {
     var viewTd = getEl('td');
     viewTd.style.textAlign  = 'center';
     if (statusC == 'Finished') {
-        var a = getEl('a');
-        a.setAttribute('href', '/result/index.html?job_id=' + job.id)
-        a.setAttribute('target', '_blank');
-        var button = getEl('button');
-        addEl(button, getTn('Open Result Viewer'));
-        button.classList.add('butn');
-        button.classList.add('launch-button');
-        button.disabled = !job.viewable;
-        addEl(a, button);
-        addEl(viewTd, a);
+        if (job.result_available) {
+            var a = getEl('a');
+            a.setAttribute('href', '/result/index.html?job_id=' + job.id)
+            a.setAttribute('target', '_blank');
+            var button = getEl('button');
+            addEl(button, getTn('Open Result Viewer'));
+            button.classList.add('butn');
+            button.classList.add('launch-button');
+            button.disabled = !job.viewable;
+            addEl(a, button);
+            addEl(viewTd, a);
+        } else {
+            var button = getEl('button');
+            button.textContent = 'Update to View';
+            button.classList.add('butn');
+            button.classList.add('launch-button');
+            button.disabled = !job.viewable;
+            button.setAttribute('job_id', job.id);
+            button.addEventListener('click', function (evt) {
+                this.textContent = 'Updating DB...';
+                var jobId = this.getAttribute('job_id');
+                $.ajax({
+                    url: '/submit/updateresultdb',
+                    type: 'GET',
+                    data: {job_id: jobId},
+                    success: function (response) {
+                        showJobListPage();
+                    }
+                });
+
+            });
+            addEl(viewTd, button);
+        }
     } else {
         viewTd.textContent = statusC;
     }
@@ -376,7 +410,7 @@ function populateJobTr (job) {
     // Reports
     for (var i = 0; i < GLOBALS.reports.valid.length; i++) {
         var reportType = GLOBALS.reports.valid[i];
-        if (reportType == 'text') {
+        if (reportType == 'text' || reportType == 'pandas' || reportType == 'stdout') {
             continue;
         }
         var btn = getEl('button');
@@ -468,14 +502,6 @@ function populateJobDetailTr (job) {
     var tbody = getEl('tbody');
     addEl(detailTable, tbody);
     var tr = getEl('tr');
-    var td = getEl('td');
-    td.style.width = '150px';
-    td.textContent = 'Job ID';
-    addEl(tr, td);
-    var td = getEl('td');
-    td.textContent = ji;
-    addEl(tr, td);
-    addEl(tbody, tr);
     if (job.open_cravat_version != undefined) {
         var tr = getEl('tr');
         var td = getEl('td');
@@ -488,19 +514,20 @@ function populateJobDetailTr (job) {
     }
     var tr = getEl('tr');
     var td = getEl('td');
+    td.style.width = '160px';
     td.textContent = 'Annotators';
     addEl(tr, td);
     var td = getEl('td');
     td.textContent = annotVerStr;
     addEl(tr, td);
     addEl(tbody, tr);
-    if (job.num_input_var != undefined) {
+    if (job.num_unique_var != undefined) {
         var tr = getEl('tr');
         var td = getEl('td');
-        td.textContent = '# input variants';
+        td.textContent = '# unique input variants';
         addEl(tr, td);
         var td = getEl('td');
-        td.textContent = job.num_input_var;
+        td.textContent = job.num_unique_var;
         addEl(tr, td);
         addEl(tbody, tr);
     }
@@ -1829,6 +1856,14 @@ function populateMultInputsMessage() {
         var $fileListDiv = $('#mult-inputs-list');
         for (var i=0; i<files.length; i++) {
             var file = files[i];
+            if (file.name.indexOf(' ') > -1) {
+                var alertDiv = getEl('div');
+                var span = getEl('span');
+                span.textContent = 'Space in file names is not allowed.';
+                addEl(alertDiv, span);
+                showYesNoDialog(alertDiv, null, false, true);
+                return;
+            }
             if (inputFileList.indexOf(file.name) == -1) {
                 var sdiv = getEl('div');
                 var span = getEl('span');
