@@ -11,7 +11,6 @@ import sys
 import traceback
 import shutil
 from aiohttp import web
-import aiosqlite3
 import hashlib
 from distutils.version import LooseVersion
 import glob
@@ -26,6 +25,8 @@ from cravat import constants
 from cravat import get_live_annotator, get_live_mapper
 import signal
 import gzip
+from cravat.cravat_util import max_version_supported_for_migration
+import cravat.util
 
 cfl = ConfigLoader()
 
@@ -91,7 +92,7 @@ class FileRouter(object):
                 job_dir = os.path.join(jobs_dirs[0], job_id)
         return job_dir
 
-    async def job_input(self, request, job_id):
+    async def job_input (self, request, job_id):
         job_dir, statusjson = await self.job_status(request, job_id)
         orig_input_fname = None
         if 'orig_input_fname' in statusjson:
@@ -103,7 +104,7 @@ class FileRouter(object):
                     orig_input_fname = fn[:-4]
                     break
         if orig_input_fname is not None:
-            orig_input_path = os.path.join(job_dir, orig_input_fname)
+            orig_input_path = [os.path.join(job_dir, v) for v in orig_input_fname]
         else:
             orig_input_path = None
         return orig_input_path
@@ -299,11 +300,11 @@ async def submit (request):
                         )
     # Subprocess arguments
     input_fpaths = [os.path.join(job_dir, fn) for fn in input_fnames]
-    run_args = ['cravat']
+    run_args = ['oc', 'run']
     for fn in input_fnames:
         run_args.append(os.path.join(job_dir, fn))
     # Annotators
-    if 'annotators' in job_options and len(job_options['annotators']) > 0:
+    if 'annotators' in job_options and len(job_options['annotators']) > 0 and job_options['annotators'][0] != '':
         annotators = job_options['annotators']
         annotators.sort()
         run_args.append('-a')
@@ -323,7 +324,6 @@ async def submit (request):
         await cravat_multiuser.update_user_settings(request, {'lastAssembly':assembly})
     else:
         au.set_cravat_conf_prop('last_assembly', assembly)
-
     # Reports
     if 'reports' in job_options and len(job_options['reports']) > 0:
         run_args.append('-t')
@@ -333,14 +333,9 @@ async def submit (request):
     # Note
     if 'note' in job_options:
         note = job_options['note']
-    else:
-        note = ''
-    run_args.append('--note')
-    if 'note' in job_options:
-        note = job_options['note']
-    else:
-        note = ''
-    run_args.append(note)
+        if note != '':
+            run_args.append('--note')
+            run_args.append(note)
     # Forced input format
     if 'forcedinputformat' in job_options and job_options['forcedinputformat']:
         run_args.append('--input-format')
@@ -481,6 +476,15 @@ async def get_job (request, job_id):
                 existing_reports.append(report_type)
     job.set_info_values(reports=existing_reports)
     job.info['username'] = os.path.basename(os.path.dirname(job_dir))
+    if 'open_cravat_version' not in job.info:
+        job.info['open_cravat_version'] = '0.0.0'
+    if LooseVersion(job.info['open_cravat_version']) < max_version_supported_for_migration:
+        job.info['result_available'] = False
+    else:
+        job.info['result_available'] = True
+    for annot_to_del in ['extra_vcf_info', 'extra_variant_info']:
+        if annot_to_del in job.info['annotators']:
+            job.info['annotators'].remove(annot_to_del)
     return job
 
 async def get_jobs (request):
@@ -522,6 +526,8 @@ async def get_all_jobs (request):
         direntries = [de for de in dir_it]
         de_names = []
         for it in direntries:
+            if it.name.startswith('.'):
+                continue
             de_names.append(it.name)
         all_jobs.extend(de_names)
     all_jobs.sort(reverse=True)
@@ -583,14 +589,12 @@ async def generate_report(request):
     global filerouter
     job_id = request.match_info['job_id']
     report_type = request.match_info['report_type']
-    job_input = await filerouter.job_run_path(request, job_id)
-    run_args = ['oc', 'run', job_input]
-    run_args.extend(['--startat', 'reporter'])
-    run_args.extend(['--repeat', 'reporter'])
+    job_db_path = await filerouter.job_db(request, job_id)
+    run_args = ['oc', 'report', job_db_path]
     run_args.extend(['-t', report_type])
-    run_args.extend(['-l', 'hg38']) # dummy -l option
     p = await asyncio.create_subprocess_shell(' '.join(run_args))
     await p.wait()
+    #await p.terminate()
     return web.json_response('done')
 
 async def download_report(request):
@@ -867,10 +871,9 @@ def fetch_job_queue (job_queue, run_jobs_info):
 
     main_loop = asyncio.new_event_loop()
     job_tracker = JobTracker(main_loop)
-    try:
-        main_loop.run_until_complete(job_worker_main())
-    except KeyboardInterrupt:
-        pass
+    main_loop.run_until_complete(job_worker_main())
+    job_tracker.loop.close()
+    main_loop.close()
 
 async def redirect_to_index (request):
     global servermode
@@ -951,7 +954,7 @@ async def live_annotate (input_data, annotators):
     global live_modules
     global live_mapper
     response = {}
-    crx_data, alt_transcripts = live_mapper.map(input_data)
+    crx_data = live_mapper.map(input_data)
     crx_data = live_mapper.live_report_substitute(crx_data)
     crx_data[mapping_parser_name] = AllMappingsParser(crx_data[all_mappings_col_name])
     for k, v in live_modules.items():
@@ -1047,6 +1050,31 @@ def get_status_json_in_dir (job_dir):
                 status_json = json.load(f)
     return status_json
 
+async def update_result_db (request):
+    queries = request.rel_url.query
+    job_id = queries['job_id']
+    global filerouter
+    job_dir = await filerouter.job_dir(request, job_id)
+    fns = find_files_by_ending(job_dir, '.sqlite')
+    db_path = os.path.join(job_dir, fns[0])
+    cmd = ['oc', 'util', 'update-result', db_path]
+    p = await asyncio.create_subprocess_shell(' '.join(cmd))
+    await p.wait()
+    compatible_version, db_version, oc_version = cravat.util.is_compatible_version(db_path)
+    if compatible_version:
+        msg = 'success'
+        fn = find_files_by_ending(job_dir, '.status.json')[0]
+        path = os.path.join(job_dir, fn)
+        with open(path) as f:
+            status_json = json.load(f)
+        status_json['open_cravat_version'] = str(db_version)
+        wf = open(path, 'w')
+        json.dump(status_json, wf, indent=2, sort_keys=True)
+        wf.close()
+    else:
+        msg = 'fail'
+    return web.json_response(msg)
+
 filerouter = FileRouter()
 VIEW_PROCESS = None
 live_modules = None
@@ -1084,6 +1112,7 @@ routes.append(['GET', '/submit/annotate', get_live_annotation_get])
 routes.append(['POST', '/submit/annotate', get_live_annotation_post])
 routes.append(['GET', '/', redirect_to_index])
 routes.append(['GET', '/submit/jobs/{job_id}/status', get_job_status])
+routes.append(['GET', '/submit/updateresultdb', update_result_db])
 
 if __name__ == '__main__':
     app = web.Application()
